/*****************************************************************************/

/// \file       init.c
/// \author     Frank Redlich
/// \date       07.01.2023
/// \brief      Initialisierungsroutinen
/// \details    Projekt:    Carrera_Decoder_PIC16F18424<br>
///             Prozessor:  Microchip PIC16F18424 <br>
///             Hardware:   DIY2, Decoder Mini
/// \version    1.01
///             compilerswitch fr Prozessortakt entfernt, jetzt immer 32MHz   
/// \version    1.02
///             init CCP4 fr PWM Rcklicht/Bremslicht fr DIY Decoder
/// \version    1.03
///             fr PIC16F1825 per Compilerswitch CCPTMRS statt CCPTMRS0
/// \version    1.04
///             new microcontroller PIC16F18424  
/// \version    1.05
///             bugfix: timer 2 prescaler 

/******************************************************************************/



#include <xc.h>
#include "version.h"
#include "ports.h"
#include "typedef.h"
#include "defines.h"
#include "variable.h"
#include "eeprom.h"


#include "init.h"



/**
 * initialisiert die Peripherie des PICs
 */
void Initialisierung(void)
{       
    OSCILLATOR_Initialize();
    
    
    //----------------------------------------- Ports auf Low stellen
    PORTA = 0x00; 
    PORTC = 0x00;
 
    //----------------------------------------- Ports definieren auf Eingang/Ausgang
    // 0 == Output
    // 1 == Input
    // d == digital
    // a == analog
    // p == push pull
    // D == open Drain
    // R == Pullup Resistor

    TRISA  = 0x2B;	      // --oi ioii
    WPUA   = 0x20;       // --R. ....
    ANSELA = 0x00;       // --dd dddd
   
    //TRISC  = 0x00;	      // --oo oooo
    TRISC  = 0x28;	      // --io iooo      nach Reset erstmal die Pins PWM Motor und Bremse als Eingang schalten
    WPUC   = 0x00;       // --.. ....
    ANSELC = 0x00;       // --dd dddd

    RC5PPS = 0x09;   //RC5->CCP1:CCP1;   
    RC3PPS = 0x0A;   //RC3->CCP2:CCP2;            
    RA2PPS = 0x0B;   //RA2->CCP3:CCP3;    

    
    alle_Variablen_ruecksetzen();

    Init_Timer_1();                // Zeitbasis fr Pulsweitenmessung der CU Daten
    Init_Timer_2();                // clock source for PWM3/CCP3
    Init_Timer_4();                // Takt fr CCP1 und CCP2
    Init_Timer_6();                // Timeout fr Flanken beim Datenempfang
    Init_CCP1();                   // CCP1 fr Motorsteuerung
    Init_CCP2();                   // CCP2 fr Bremse
    //Init_CCP3(IR_LED_PWM_CAR_4);   // CCP3 init later for IR LED
    
    Init_IOC_Interrupt();          // Portpin A5 Interrupts fr Messung des CU Datensignals

    
    
    INTCONbits.PEIE = 1;           // peripheral int enable
    INTCONbits.GIE = 1;            // Global interrupt enable
        
}

/******************************************************************************/




/******************************************************************************/

/**
 * timebase for measuring the pulse width of data siganl 
 * Clock: Fosc/4 = 32MHz/4 => 125 nsec
 *        Prescaler 8:1 ==> 1sec 
 */
void Init_Timer_1(void)
{
    T1CLK = 0x01;                   // CS FOSC/4; 
    T1CONbits.CKPS = 0b11;          // Prescaler = 8:1
    PIR4bits.TMR1IF = 0;            // Clearing IF flag.
    PIE4bits.TMR1IE = 0;            // disble interrupt
    T1CONbits.TMR1ON = 1;           // Timer 1 einschalten
}

/******************************************************************************/
/**
 * copied from mcc_generated_files
 * prepare timer 2 for clock source of PWM 3 to control the IR LED
 */
void Init_Timer_2(void)
{
    T2CLKCON = 0x01;            // T2CS FOSC/4; 
    T2HLT = 0x00;               // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T2RST = 0x00;               // T2RSEL T2CKIPPS pin; 
    //T2PR = 0x7F;                // T2PR 127;     
    T2PR = 0xFF;                // T2PR 255;     
    T2TMR = 0x00;               // TMR2 0; 
    PIR4bits.TMR2IF = 0;        // Clearing IF flag.
    T2CONbits.T2CKPS = 0b100;   // Prescaler = 16:1
    T2CONbits.ON = 1;           // start timer 2
}



/******************************************************************************/

/**
 * Timer 4 als Taktquelle fr die CCP1 und CCP2 (Motor und Bremse)
 * und jetzt auch fr CCP4 (Rcklicht/Bremslicht fr DIY Decoder)
 
 * * Takt: Fosc/4 = 32MHz/4 => 125 nsec
 *       Prescaler 4:1  ==> 500 nsec Takt
 
 */
void Init_Timer_4(void)
{
    T4CONbits.TMR4ON = 1;           // Timer 4 auschalten
    T4CLKCON = 0x01;                // T4CS FOSC/4; 
    T4HLT = 0x00;                   // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T4PR = 127;                     // Periodenlnge x 500ns = 64s 
    PIR4bits.TMR4IF = 0;            // Clear the TMRxIF interrupt flag bit Timer 4
    T4CONbits.T4CKPS = 0b010;       // Prescaler = 4:1
        
    T4CONbits.TMR4ON = 1;           // Timer 4 start
}


/******************************************************************************/

/**
 * Timer fr Timout beim Datenempfang
 * wird spter bei Startlcke gestartet
 * wird beendet, wenn bergelaufen
 * 
 * 32 MHz Fosc/4 = 4 MHz ==> 125ns Takt
 *        Prescaler 64:1 ==> 8s Takt
 *        Postscaler 1:1 ==> Interrupt bei 26 x 8 s = 208 sec
 */

void Init_Timer_6(void)
{
    T6CONbits.TMR6ON = 0;           // Timer 6 ausschalten
    PR6 = 26;
    TMR6 = 0;
    T6CONbits.T6CKPS = 0b110;       // Prescaler = 64:1
    T6CONbits.T6OUTPS = 0b0000;     // Postscaler = 1:1
    T6CLKCONbits.T6CS = 0b0001;     // Timer 6 clock source = Fosc/4
}


/******************************************************************************/




/**
 * Motor
 *  
 *     PWM Period = [(PRx) + 1] * 4 * TOSC * (TMRx Prescale Value)
 *     TOSC = 1/FOSC 
 * 
 * Taktquelle ist Timer 4 (wurde vorher initialisiert)
 *     Fosc/4 = 32MHz/4 = 8MHz
 *     ==> 125ns Takt
 *     Prescaler 4:1 ==> 0,5 s Takt
 */
void Init_CCP1(void)
{
    TRISCbits.TRISC5 = 1;           // Disable the CCP1 pin output driver by setting the associated TRIS bit.
	CCP1CON = 0x8C;                 // MODE PWM; EN enabled; FMT right_aligned; 	
	CCPR1H = 0x00;                  // RH 0; 		
    CCPR1L = 0x00;                  // RL 0; 
    
    CCPTMRS0bits.C1TSEL = 0b10;     // Selecting Timer 4
    
    TRISCbits.TRISC5 = 0;           // Enable CCP3 Pin
}

/******************************************************************************/


/**
 * Bremse
 * Taktquelle ist Timer 4
 *     Fosc/4 = 16MHz/4 = 4MHz
 *     ==> 250ns Takt
 *     Prescaler 1:1 ==> 0,25 s Takt
 */
void Init_CCP2(void)
{    
    TRISCbits.TRISC3 = 1;           // Disable the CCP2 pin output driver by setting the associated TRIS bit.    
    
	CCP2CON = 0x8C;                 // MODE PWM; EN enabled; FMT right_aligned; 	
	CCPR2H = 0x00;                  // RH 0; 		
    CCPR2L = 0x00;                  // RL 0; 
    
    CCPTMRS0bits.C2TSEL = 0b10;     // Selecting Timer 4
    
    TRISCbits.TRISC3 = 0;           // Enable CCP2 Pin
}

/***************************************************************************************/


/**
 * IR LED
 * Taktquelle ist Timer 2
 *     Fosc/4 = 32MHz/4 = 8MHz
 *     ==> 125ns Takt
 *     Prescaler 16:1 ==> 2s Takt
 
 */
void Init_CCP3(U8 Periodendauer)
{
    U16     dutyValue;
    
    TRISAbits.TRISA2 = 1;           // Disable the CCP3 pin output driver by setting the associated TRIS bit.

    PR2 = Periodendauer;            // Periodenlnge x 4s

    CCP3CONbits.CCP3MODE = 0b1100;  // PWM mode    
    CCP3CONbits.FMT      = 0;       // Right-aligned format  
    CCP3CONbits.EN       = 1;       // start PWM
	CCP3CON = 0x8C;                 // MODE PWM; EN enabled; FMT right_aligned; 

    dutyValue = Periodendauer * 2;
    dutyValue &= 0x03FF;
    
    // Load duty cycle value
    CCPR3H = (U8)(dutyValue >> 8);
    CCPR3L = (U8)dutyValue;
    

    CCPTMRS0bits.C3TSEL = 0x1;      // Selecting Timer 2
    
    TRISAbits.TRISA2 = 0;           // Enable CCP3 Pin
     
}


/******************************************************************************/


/**
 * Portpin Interrupts fr Messung der Startlcken einschalten
 * nur fallende Flanke
 */
void Init_IOC_Interrupt(void)
{
    IOCAPbits.IOCAP5 = 1;   // RA5 Data steigende Flanke enable
    IOCANbits.IOCAN5 = 1;   // RA5 Data fallende Flanke enable 
    
    PIE0bits.IOCIE = 1;     // Enable interrupt on change
} 


/******************************************************************************/             
             
/**
 * setzt alle gloabale Variable zurck
 */
void alle_Variablen_ruecksetzen(void)
{
    U8      Index;


    Telegramm_empfangen = NEIN;
    Licht_Sperr_Timer = 0;
    Ghostcar_Doppelklick_timeout = 0;
    Ghostcar_Doppelklick_Counter = 0;
    Bremslicht_timer = 0;
    Tank_leer_Blinken = NEIN;
    Tank_leer = NEIN;

    CU_Prg_Daten.Parameter_alt = PRG_NORMALBETRIEB;
    CU_Prg_Daten.Wert_alt = 0;

    Adr_warte_auf_Reset_timeout = 0;

    for (Index = 0; Index < MAX_REGLER; Index++)
    {
        Spurwechseltaste.Counter[Index] = 0;
        Spurwechseltaste.Letzter_Zustand[Index] = LOSGELASSEN;
    }
}

/******************************************************************************/

/**
 * prft ob die Programmierwerte wie z.B. die Autoadresse schon beschrieben
 * wurden oder ob immer noch berall FF drin steht. Wenn ja, dann werden berall
 * Null als Defaultwerte geschrieben
 * 
 * Die Adressen EE_ADR_DEFAULT_0XAA und EE_ADR_DEFAULT_0X55 gelten als Flag ob
 * die Werte schon geschrieben wurden.
 */
void Init_EEPROM_Defaultwerte(void)
{
    U8 value;
            
    value = EE_read(EE_ADR_DEFAULT_0XAA);
    if (value != 0xAA)                        
    //if (EE_read(EE_ADR_DEFAULT_0XAA) != 0xAA)
    {
        value = EE_read(EE_ADR_DEFAULT_0X55);
        if (value != 0x55)                                
        //if (EE_read(EE_ADR_DEFAULT_0X55) != 0x55)
        {        
            EE_write_verified(0, EE_ADR_AUTO_ADRESSE);              // Auto 1 = Adresse 0            
            EE_write_verified(1, EE_ADR_LICHT);                     // Licht an
            EE_write_verified(0, EE_ADR_PROGRAMMIERMODUS);          // aus
            EE_write_verified(6, EE_ADR_BREMSWIRKUNG);              // minimale Bremswirkung
            EE_write_verified(1, EE_ADR_MAX_GESCHWINDIGKEIT);       // langsame Endgeschwindigkeit
            EE_write_verified(0, EE_ADR_GHOST_GESCHWINDIGKEIT);
            EE_write_verified(0, EE_ADR_GHOST_PROGRAMMIERMODUS);

            EE_write_verified(0xAA, EE_ADR_DEFAULT_0XAA);           // Flag, dass Default-Werte drin sind
            EE_write_verified(0x55, EE_ADR_DEFAULT_0X55);
        }
    }
}

/******************************************************************************/

/**
 * set up the oscilator frequencies
 * copied from mcc_generated files
 * Fosc = 32MHz
 */
void OSCILLATOR_Initialize(void)
{    
    OSCCON1 = 0x60;     // NOSC HFINTOSC; NDIV 1; 
    OSCCON3 = 0x00;     // CSWHOLD may proceed; SOSCPWR Low power;     
    OSCEN   = 0x00;     // MFOEN disabled; LFOEN disabled; ADOEN disabled; SOSCEN disabled; EXTOEN disabled; HFOEN disabled;     
    OSCFRQ  = 0x06;     // 
    OSCTUNE = 0x00;     // HFTUN 0; 
}

