/*****************************************************************************/

/// \file       EEPROM.c
/// \author     Frank Redlich
/// \date       12.11.2022
/// \brief      Zugriff auf das interne EEPROM
/// \details    Projekt:    Carrera_Decoder_PIC16F18424<br>
///             Prozessor:  Microchip PIC16F18424 <br>
///             Hardware:   DIY2, Decoder Mini
/// \version    1.01
///             kein Blinken mehr wenn irgendeine EEPROM Zelle beschrieben wird 

/// \version    1.02
///             new microcontrolle PIC16F18424

/******************************************************************************/


#include <xc.h>
#include "ports.h"
#include "init.h"
#include "typedef.h"
#include "defines.h"
#include "variable.h"

#include "eeprom.h"






/**
 * liest ein Byte aus dem internen EEPROM
 * 
 * @param Adresse   Adresse im EEPROM 0 ... 255
 * @return          ausgelesener Wert
 */
U8 EE_read (U8 address)
{
    uint16_t    bAdd;
    
    bAdd = (U16)address | 0x7000;   // start address of eeprom in this PIC

    NVMADRH = ((bAdd >> 8) & 0xFF);
    NVMADRL = (bAdd & 0xFF);
    NVMCON1bits.NVMREGS = 1;    
    NVMCON1bits.RD = 1;
    NOP();  // NOPs may be required for latency at high frequencies
    NOP();
    
    return (NVMDATL);
}

/******************************************************************************/

/**
 * Schreiben mit Verify eines Bytes im EEPROM
 *
 * @param Databyte      Daten Byte
 * @param Adresse       Adresse im EEPROM 0 ... 255
 * @return              OK wenn verify fehlerfrei, sonst NOK
 */
U8 EE_write_verified (U8 Databyte, U8 Adresse)
{
    U8      Ergebnis,
            Versuche,
            Testbyte;


    Versuche = 5;
    Ergebnis = NOK;
    while ((Versuche > 0) && (Ergebnis == NOK))
    {
         EE_write (Databyte, Adresse);
         Testbyte = EE_read (Adresse);
         if (Testbyte == Databyte)
             Ergebnis = OK;
         Versuche--;
    }
    
    
    return(Ergebnis);
}


/******************************************************************************/


/**
 * schreibt ein Byte in das interne EEPROM
 * ohne verify
 * 
 * @param Databyte      Daten Byte
 * @param Adresse       Adresse im EEPROM 0 ... 255
 */
void EE_write (U8 Databyte, U8 address)
{
    uint8_t     GIEBitValue = INTCONbits.GIE;
    uint16_t    bAdd;
    
    bAdd = (U16)address | 0x7000;   // start address of eeprom in this PIC

    NVMADRH = ((bAdd >> 8) & 0xFF);
    NVMADRL = (bAdd & 0xFF);
    NVMDATL = Databyte;    
    NVMCON1bits.NVMREGS = 1;
    NVMCON1bits.WREN = 1;
    INTCONbits.GIE = 0;     // Disable interrupts
    NVMCON2 = 0x55;
    NVMCON2 = 0xAA;
    NVMCON1bits.WR = 1;
    // Wait for write to complete
    while (NVMCON1bits.WR)
    {
    }

    NVMCON1bits.WREN = 0;
    INTCONbits.GIE = GIEBitValue;   // restore interrupt enable
    
    
    
}

/******************************************************************************/

